@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2019 gnome-mpv
+ * Copyright (c) 2017-2020 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -177,6 +177,9 @@ video_area_resize_handler(	CelluloidView *view,
 				gint height,
 				gpointer data );
 
+static void
+searching_handler(GObject *object, GParamSpec *pspec, gpointer data);
+
 static void
 fullscreen_handler(GObject *object, GParamSpec *pspec, gpointer data);
 
@@ -210,6 +213,9 @@ celluloid_controller_class_init(CelluloidControllerClass *klass);
 static void
 celluloid_controller_init(CelluloidController *controller);
 
+static void
+update_extra_mpv_options(CelluloidController *controller);
+
 G_DEFINE_TYPE(CelluloidController, celluloid_controller, G_TYPE_OBJECT)
 
 static void
@@ -235,6 +241,7 @@ constructed(GObject *object)
 	connect_signals(controller);
 	celluloid_controller_action_register_actions(controller);
 	celluloid_controller_input_connect_signals(controller);
+	update_extra_mpv_options(controller);
 
 	gtk_widget_show_all(GTK_WIDGET(window));
 
@@ -411,23 +418,13 @@ static void
 view_ready_handler(CelluloidView *view, gpointer data)
 {
 	CelluloidController *controller = CELLULOID_CONTROLLER(data);
-	CelluloidApplication *app = controller->app;
 	CelluloidModel *model = controller->model;
-	GSettings *settings = g_settings_new(CONFIG_ROOT);
-	const gchar *cli_options = celluloid_application_get_mpv_options(app);
-	gchar *pref_options = g_settings_get_string(settings, "mpv-options");
-	gchar *options = g_strjoin(" ", pref_options, cli_options, NULL);
 
 	celluloid_player_options_init
 		(	CELLULOID_PLAYER(controller->model),
 			CELLULOID_MAIN_WINDOW(controller->view) );
 
-	g_object_set(model, "extra-options", options, NULL);
-	celluloid_model_initialize(model, options);
-
-	g_free(options);
-	g_free(pref_options);
-	g_object_unref(settings);
+	celluloid_model_initialize(model);
 }
 
 static void
@@ -449,7 +446,11 @@ render_handler(CelluloidView *view, gpointer data)
 static void
 preferences_updated_handler(CelluloidView *view, gpointer data)
 {
-	celluloid_model_reset(CELLULOID_CONTROLLER(data)->model);
+	CelluloidController *controller = data;
+
+	update_extra_mpv_options(controller);
+	celluloid_view_make_gl_context_current(controller->view);
+	celluloid_model_reset(controller->model);
 }
 
 static void
@@ -569,6 +570,9 @@ connect_signals(CelluloidController *controller)
 	g_object_bind_property(	controller->model, "fullscreen",
 				controller->view, "fullscreen",
 				G_BINDING_BIDIRECTIONAL );
+	g_object_bind_property(	controller->model, "window-maximized",
+				controller->view, "maximized",
+				G_BINDING_BIDIRECTIONAL );
 	g_object_bind_property(	controller->model, "pause",
 				controller->view, "pause",
 				G_BINDING_DEFAULT );
@@ -578,6 +582,9 @@ connect_signals(CelluloidController *controller)
 	g_object_bind_property(	controller->model, "volume",
 				controller->view, "volume",
 				G_BINDING_BIDIRECTIONAL );
+	g_object_bind_property(	controller->model, "volume-max",
+				controller->view, "volume-max",
+				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "duration",
 				controller->view, "duration",
 				G_BINDING_DEFAULT );
@@ -593,11 +600,12 @@ connect_signals(CelluloidController *controller)
 	g_object_bind_property(	controller->view, "display-fps",
 				controller->model, "display-fps",
 				G_BINDING_DEFAULT|G_BINDING_SYNC_CREATE );
-	g_object_bind_property_full(	controller->model, "loop-playlist",
-					controller->view, "loop",
-					G_BINDING_BIDIRECTIONAL,
-					loop_to_boolean,
+	g_object_bind_property_full(	controller->view, "loop",
+					controller->model, "loop-playlist",
+					G_BINDING_BIDIRECTIONAL|
+					G_BINDING_SYNC_CREATE,
 					boolean_to_loop,
+					loop_to_boolean,
 					NULL,
 					NULL );
 
@@ -658,6 +666,10 @@ connect_signals(CelluloidController *controller)
 				"notify::fullscreen",
 				G_CALLBACK(fullscreen_handler),
 				controller );
+	g_signal_connect(	controller->view,
+				"notify::searching",
+				G_CALLBACK(searching_handler),
+				controller );
 	g_signal_connect(	controller->view,
 				"button-clicked::play",
 				G_CALLBACK(play_button_handler),
@@ -1050,6 +1062,16 @@ fullscreen_handler(GObject *object, GParamSpec *pspec, gpointer data)
 		(G_SIMPLE_ACTION(toggle_controls), !fullscreen);
 }
 
+static void
+searching_handler(GObject *object, GParamSpec *pspec, gpointer data)
+{
+	// When the search box becomes visible, it blocks all keyboard inputs
+	// from being handled by CelluloidController. This means that the key up
+	// event for the key that triggered the search will never arrive, so we
+	// need to explicitly reset key states here.
+	celluloid_model_reset_keys(CELLULOID_CONTROLLER(data)->model);
+}
+
 static void
 play_button_handler(GtkButton *button, gpointer data)
 {
@@ -1206,6 +1228,23 @@ celluloid_controller_init(CelluloidController *controller)
 	controller->mpris = NULL;
 }
 
+static void
+update_extra_mpv_options(CelluloidController *controller)
+{
+	GSettings *settings =		g_settings_new(CONFIG_ROOT);
+	const gchar *cli_options =	celluloid_application_get_mpv_options
+					(controller->app);
+	gchar *pref_options =		g_settings_get_string(settings, "mpv-options");
+	gchar *options =		g_strjoin(" ", pref_options, cli_options, NULL);
+
+
+	g_object_set(controller->model, "extra-options", options, NULL);
+
+	g_free(options);
+	g_free(pref_options);
+	g_object_unref(settings);
+}
+
 CelluloidController *
 celluloid_controller_new(CelluloidApplication *app)
 {
