@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 gnome-mpv
+ * Copyright (c) 2014-2020 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -322,6 +322,9 @@ celluloid_main_window_init(CelluloidMainWindow *wnd)
 	g_object_bind_property(	priv->control_box, "skip-enabled",
 				vid_area_control_box, "skip-enabled",
 				G_BINDING_DEFAULT );
+	g_object_bind_property(	priv->control_box, "show-fullscreen-button",
+				vid_area_control_box, "show-fullscreen-button",
+				G_BINDING_DEFAULT );
 	g_object_bind_property(	priv->control_box, "time-position",
 				vid_area_control_box, "time-position",
 				G_BINDING_DEFAULT );
@@ -501,25 +504,33 @@ celluloid_main_window_save_state(CelluloidMainWindow *wnd)
 	CelluloidMainWindowPrivate *priv;
 	gint width;
 	gint height;
+	gboolean maximized;
 	gint handle_pos;
 	gdouble volume;
+	gboolean loop;
 	gboolean controls_visible;
 
 	settings = g_settings_new(CONFIG_WIN_STATE);
 	priv = get_private(wnd);
+	maximized = gtk_window_is_maximized(GTK_WINDOW(wnd));
 	handle_pos = gtk_paned_get_position(GTK_PANED(priv->vid_area_paned));
-	volume =	celluloid_control_box_get_volume
-			(CELLULOID_CONTROL_BOX(priv->control_box));
 	controls_visible = gtk_widget_get_visible(priv->control_box);
 
+	g_object_get(priv->control_box, "volume", &volume, "loop", &loop, NULL);
 	gtk_window_get_size(GTK_WINDOW(wnd), &width, &height);
 
-	g_settings_set_int(settings, "width", width);
-	g_settings_set_int(settings, "height", height);
-	g_settings_set_double(settings, "volume", volume);
+	g_settings_set_boolean(settings, "maximized", maximized);
+	g_settings_set_double(settings, "volume", volume/100.0);
+	g_settings_set_boolean(settings, "loop-playlist", loop);
 	g_settings_set_boolean(settings, "show-controls", controls_visible);
 	g_settings_set_boolean(settings, "show-playlist", priv->playlist_visible);
 
+	if(!maximized)
+	{
+		g_settings_set_int(settings, "width", width);
+		g_settings_set_int(settings, "height", height);
+	}
+
 	if(celluloid_main_window_get_playlist_visible(wnd))
 	{
 		g_settings_set_int(	settings,
@@ -545,9 +556,11 @@ celluloid_main_window_load_state(CelluloidMainWindow *wnd)
 		CelluloidMainWindowPrivate *priv = get_private(wnd);
 		gint width = g_settings_get_int(settings, "width");
 		gint height = g_settings_get_int(settings, "height");
+		gboolean maximized = g_settings_get_boolean(settings, "maximized");
 		gint handle_pos;
 		gboolean controls_visible;
 		gdouble volume;
+		gboolean loop;
 
 		priv->playlist_width
 			= g_settings_get_int(settings, "playlist-width");
@@ -556,16 +569,25 @@ celluloid_main_window_load_state(CelluloidMainWindow *wnd)
 		controls_visible
 			= g_settings_get_boolean(settings, "show-controls");
 		volume = g_settings_get_double(settings, "volume");
+		loop = g_settings_get_boolean(settings, "loop-playlist");
 		handle_pos = width-(priv->playlist_visible?priv->playlist_width:0);
 
-		celluloid_control_box_set_volume
-			(CELLULOID_CONTROL_BOX(priv->control_box), volume);
+		g_object_set(	priv->control_box,
+				"volume", volume,
+				"loop", loop,
+				NULL);
+
 		gtk_widget_set_visible(priv->control_box, controls_visible);
 		gtk_widget_set_visible(priv->playlist, priv->playlist_visible);
 		gtk_window_resize(GTK_WINDOW(wnd), width, height);
 		gtk_paned_set_position
 			(GTK_PANED(priv->vid_area_paned), handle_pos);
 
+		if(maximized)
+		{
+			gtk_window_maximize(GTK_WINDOW(wnd));
+		}
+
 		g_clear_object(&settings);
 	}
 	else
